'use strict';

var tslib = require('tslib');
var React = require('react');
var filterBarStateReducer = require('./reducer/filterBarStateReducer.cjs');
var setupFilterBarState = require('./reducer/setupFilterBarState.cjs');
var checkShouldUpdateValues = require('./utils/checkShouldUpdateValues.cjs');
var createFiltersHash = require('./utils/createFiltersHash.cjs');
var getInactiveFilters = require('./utils/getInactiveFilters.cjs');
var getMappedFilters = require('./utils/getMappedFilters.cjs');
var getValidValue = require('./utils/getValidValue.cjs');
function _interopDefault(e) {
  return e && e.__esModule ? e : {
    default: e
  };
}
var React__default = /*#__PURE__*/_interopDefault(React);
var FilterBarContext = React__default.default.createContext(null);
var useFilterBarContext = function () {
  var context = React.useContext(FilterBarContext);
  if (!context) {
    throw new Error('useFilterBarContext must be used within the FilterBarContext.Provider');
  }
  return context;
};
var FilterBarProvider = function (_a) {
  var children = _a.children,
    filters = _a.filters,
    values = _a.values,
    onValuesChange = _a.onValuesChange;
  var filtersHash = React.useRef(createFiltersHash.createFiltersHash(filters));
  // As `filters` contains components, to prevent unnecessary re-renders, we check only for changes to id and name
  // eslint-disable-next-line react-hooks/exhaustive-deps
  var mappedFilters = React.useMemo(function () {
    return getMappedFilters.getMappedFilters(filters);
  }, [filtersHash.current]);
  var _b = React.useReducer(filterBarStateReducer.filterBarStateReducer, setupFilterBarState.setupFilterBarState(filters, values)),
    state = _b[0],
    dispatch = _b[1];
  var activeFilters = Array.from(state.activeFilterIds, function (id) {
    return mappedFilters[id];
  });
  // Workaround for DateRangePicker populating the values object before the value is valid
  // (it purposefully persists a state with a 'from' date but no 'to' date, but hides it on the filter button)
  var isDraftDateRange = function (v) {
    return v && v.from !== undefined && v.to === undefined;
  };
  var hasDraftDateRangeOnly = Object.values(values).every(isDraftDateRange);
  var isClearable = Object.keys(values).length > 0 && !hasDraftDateRangeOnly || state.hasRemovableFilter && activeFilters.some(function (f) {
    return f.isRemovable;
  });
  var value = {
    getFilterState: function (id) {
      return tslib.__assign(tslib.__assign({}, state.filters[id]), {
        isActive: state.activeFilterIds.has(id),
        value: values[id]
      });
    },
    isClearable: isClearable,
    getActiveFilterValues: function () {
      return values;
    },
    toggleOpenFilter: function (id, isOpen) {
      dispatch({
        type: 'update_single_filter',
        id: id,
        data: {
          isOpen: isOpen
        }
      });
    },
    setFilterOpenState: function (id, isOpen) {
      dispatch({
        type: 'update_single_filter',
        id: id,
        data: {
          isOpen: isOpen
        }
      });
    },
    openFilter: function (id) {
      dispatch({
        type: 'update_single_filter',
        id: id,
        data: {
          isOpen: true
        }
      });
    },
    updateValue: function (id, newValue) {
      var _a;
      dispatch({
        type: 'update_values',
        values: tslib.__assign(tslib.__assign({}, values), (_a = {}, _a[id] = getValidValue.getValidValue(newValue), _a))
      });
    },
    showFilter: function (id) {
      dispatch({
        type: 'activate_filter',
        id: id
      });
      dispatch({
        type: 'set_focus',
        id: id
      });
    },
    hideFilter: function (id) {
      dispatch({
        type: 'deactivate_filter',
        id: id
      });
      dispatch({
        type: 'set_focus',
        id: 'add_filter'
      });
    },
    getInactiveFilters: function () {
      return getInactiveFilters.getInactiveFilters(state);
    },
    clearAllFilters: function () {
      state.activeFilterIds.forEach(function (id) {
        if (mappedFilters[id].isRemovable) dispatch({
          type: 'deactivate_filter',
          id: id
        });
      });
      dispatch({
        type: 'update_values',
        values: {}
      });
    },
    setFocus: function (id) {
      dispatch({
        type: 'set_focus',
        id: id
      });
    },
    focusId: state.focusId
  };
  React.useEffect(function () {
    var shouldUpdate = checkShouldUpdateValues.checkShouldUpdateValues(state, values);
    if (shouldUpdate) dispatch({
      type: 'update_values',
      values: tslib.__assign({}, values)
    });
    // We only want to run this effect when the passed in values change (external event updating the values)
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [values]);
  React.useEffect(function () {
    if (state.hasUpdatedValues) {
      onValuesChange(tslib.__assign({}, state.values));
      dispatch({
        type: 'complete_update_values'
      });
    }
    // We only want to run this effect when the state has updated values
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [state]);
  React.useEffect(function () {
    var newFiltersHash = createFiltersHash.createFiltersHash(filters);
    if (newFiltersHash !== filtersHash.current) {
      filtersHash.current = newFiltersHash;
      dispatch({
        type: 'update_filter_labels',
        data: filters
      });
    }
  }, [filters]);
  return React__default.default.createElement(FilterBarContext.Provider
  // @note: Context object cannot be generic, thus the type-casting to a looser type
  , {
    // @note: Context object cannot be generic, thus the type-casting to a looser type
    value: value
  }, children(activeFilters));
};
exports.FilterBarProvider = FilterBarProvider;
exports.useFilterBarContext = useFilterBarContext;
