import { __rest, __assign } from 'tslib';
import React, { useId, useRef, useState, useEffect } from 'react';
import { useIntl } from '@cultureamp/i18n-react-intl';
import { FocusOn } from 'react-focus-on';
import { CalendarSingle } from '../Calendar/CalendarSingle/CalendarSingle.mjs';
import '../Calendar/CalendarRange/CalendarRange.mjs';
import { calculateDisabledDays } from '../Calendar/utils/calculateDisabledDays.mjs';
import { formatDateAsNumeral } from '../Calendar/utils/formatDateAsNumeral.mjs';
import { formatDateAsText } from '../Calendar/utils/formatDateAsText.mjs';
import { isDisabledDate } from '../Calendar/utils/isDisabledDate.mjs';
import { isInvalidDate } from '../Calendar/utils/isInvalidDate.mjs';
import { isSelectingDayInCalendar } from '../Calendar/utils/isSelectingDayInCalendar.mjs';
import { parseDateAsTextOrNumeral } from '../Calendar/utils/parseDateAsTextOrNumeral.mjs';
import 'date-fns';
import { setFocusInCalendar } from '../Calendar/utils/setFocusInCalendar.mjs';
import { CalendarPopover } from '../Calendar/CalendarPopover/CalendarPopover.mjs';
import { VisuallyHidden } from '../VisuallyHidden/VisuallyHidden.mjs';
import { DateInputField } from './subcomponents/DateInputField/DateInputField.mjs';
import { getLocale } from './utils/getLocale.mjs';
import { validateDate } from './utils/validateDate.mjs';

/**
 * {@link https://cultureamp.atlassian.net/wiki/spaces/DesignSystem/pages/3082061174/Date+Picker Guidance} |
 * {@link https://cultureamp.design/?path=/docs/components-date-controls-datepicker--docs Storybook}
 */
const DatePicker = /*#__PURE__*/function () {
  const DatePicker = function (_a) {
    var propsId = _a.id,
      propsButtonRef = _a.buttonRef,
      _b = _a.locale,
      propsLocale = _b === void 0 ? 'en-AU' : _b,
      disabledDates = _a.disabledDates,
      disabledDaysOfWeek = _a.disabledDaysOfWeek,
      disabledRange = _a.disabledRange,
      disabledBeforeAfter = _a.disabledBeforeAfter,
      disabledBefore = _a.disabledBefore,
      disabledAfter = _a.disabledAfter,
      weekStartsOn = _a.weekStartsOn,
      defaultMonth = _a.defaultMonth,
      selectedDay = _a.selectedDay,
      status = _a.status,
      validationMessage = _a.validationMessage,
      onInputClick = _a.onInputClick,
      onInputFocus = _a.onInputFocus,
      onInputChange = _a.onInputChange,
      onInputBlur = _a.onInputBlur,
      onButtonClick = _a.onButtonClick,
      onDayChange = _a.onDayChange,
      onValidate = _a.onValidate,
      restDateInputFieldProps = __rest(_a, ["id", "buttonRef", "locale", "disabledDates", "disabledDaysOfWeek", "disabledRange", "disabledBeforeAfter", "disabledBefore", "disabledAfter", "weekStartsOn", "defaultMonth", "selectedDay", "status", "validationMessage", "onInputClick", "onInputFocus", "onInputChange", "onInputBlur", "onButtonClick", "onDayChange", "onValidate"]);
    var formatMessage = useIntl().formatMessage;
    var calendarLabelDesc = formatMessage({
      id: 'datePicker.calendarLabelDescription',
      defaultMessage: 'Select date from calendar for:',
      description: 'Label for the search input'
    });
    var reactId = useId();
    var id = propsId !== null && propsId !== void 0 ? propsId : reactId;
    var containerRef = useRef(null);
    var inputRef = useRef(null);
    var fallbackButtonRef = useRef(null);
    var buttonRef = propsButtonRef !== null && propsButtonRef !== void 0 ? propsButtonRef : fallbackButtonRef;
    var dateInputRefs = useRef({
      inputRef: inputRef,
      buttonRef: buttonRef
    });
    var _c = useState(''),
      inputValue = _c[0],
      setInputValue = _c[1];
    var _d = useState(false),
      isOpen = _d[0],
      setIsOpen = _d[1];
    var _e = useState(),
      lastTrigger = _e[0],
      setLastTrigger = _e[1];
    var _f = useState(),
      inbuiltStatus = _f[0],
      setInbuiltStatus = _f[1];
    var _g = useState(),
      inbuiltValidationMessage = _g[0],
      setInbuiltValidationMessage = _g[1];
    var shouldUseInbuiltValidation = onValidate === undefined;
    var locale = getLocale(propsLocale);
    var disabledDays = calculateDisabledDays({
      disabledDates: disabledDates,
      disabledDaysOfWeek: disabledDaysOfWeek,
      disabledRange: disabledRange,
      disabledBeforeAfter: disabledBeforeAfter,
      disabledBefore: disabledBefore,
      disabledAfter: disabledAfter
    });
    var handleValidation = function (validationResponse) {
      if (shouldUseInbuiltValidation) {
        setInbuiltStatus(validationResponse.status);
        setInbuiltValidationMessage(validationResponse.validationMessage);
      } else {
        onValidate(validationResponse);
      }
    };
    var handleDayChange = function (date, newInputValue) {
      var _a = validateDate({
          date: date,
          inputValue: newInputValue,
          disabledDays: disabledDays
        }),
        validationResponse = _a.validationResponse,
        newDate = _a.newDate;
      handleValidation(validationResponse);
      onDayChange(newDate);
    };
    var handleCalendarDayChange = function (date) {
      if (!isDisabledDate(date, disabledDays)) {
        var newInputValue = lastTrigger === 'calendarButton' ? formatDateAsText(date, disabledDays, locale) : formatDateAsNumeral(date, locale);
        setInputValue(newInputValue);
        handleDayChange(date);
        setIsOpen(false);
      }
    };
    var handleInputClick = function (e) {
      setIsOpen(true);
      onInputClick === null || onInputClick === void 0 ? void 0 : onInputClick(e);
    };
    var handleInputFocus = function (e) {
      setLastTrigger('inputFocus');
      if (selectedDay) {
        var newInputValue = formatDateAsNumeral(selectedDay, locale);
        setInputValue(newInputValue);
      }
      onInputFocus === null || onInputFocus === void 0 ? void 0 : onInputFocus(e);
    };
    var handleInputChange = function (e) {
      setInputValue(e.target.value);
      onInputChange === null || onInputChange === void 0 ? void 0 : onInputChange(e);
    };
    var handleInputBlur = function (e) {
      if (isSelectingDayInCalendar(e.relatedTarget)) return;
      if (inputValue !== '') {
        var parsedDate = parseDateAsTextOrNumeral(inputValue, locale);
        if (!isInvalidDate(parsedDate)) {
          setInputValue(formatDateAsText(parsedDate, disabledDays, locale));
        }
        handleDayChange(parsedDate, inputValue);
        onInputBlur === null || onInputBlur === void 0 ? void 0 : onInputBlur(e);
        return;
      }
      handleDayChange(undefined, inputValue);
      onInputBlur === null || onInputBlur === void 0 ? void 0 : onInputBlur(e);
    };
    var handleKeyDown = function (e) {
      if (e.key === 'Enter') {
        setIsOpen(false);
        var parsedDate = parseDateAsTextOrNumeral(inputValue, locale);
        handleDayChange(parsedDate, e.currentTarget.value);
      }
      if (e.key === 'ArrowDown' || e.key === 'ArrowDown' && e.altKey === true) {
        e.preventDefault();
        setIsOpen(true);
        setLastTrigger('inputKeydown');
      }
    };
    var handleButtonClick = function (e) {
      setIsOpen(!isOpen);
      setLastTrigger('calendarButton');
      onButtonClick === null || onButtonClick === void 0 ? void 0 : onButtonClick(e);
    };
    var handleCalendarMount = function (calendarElement) {
      if (lastTrigger === 'inputFocus') return;
      setFocusInCalendar(calendarElement, selectedDay);
    };
    var handleReturnFocus = function () {
      var _a, _b;
      if (lastTrigger === 'inputKeydown' || lastTrigger === 'inputFocus') {
        return (_a = inputRef.current) === null || _a === void 0 ? void 0 : _a.focus();
      }
      (_b = buttonRef.current) === null || _b === void 0 ? void 0 : _b.focus();
    };
    useEffect(function () {
      if (selectedDay) {
        setInputValue(formatDateAsText(selectedDay, disabledDays, locale));
        var formattedDate = formatDateAsNumeral(selectedDay, locale);
        var validationResponse = validateDate({
          date: selectedDay,
          inputValue: formattedDate,
          disabledDays: disabledDays
        }).validationResponse;
        if (!validationResponse.isValidDate && !validationResponse.isEmpty) {
          handleValidation(validationResponse);
        }
      }
      // @todo: Fix if possible - avoiding breaking in eslint upgrade
      // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [selectedDay]);
    var calendarId = "".concat(id, "-calendar-dialog");
    return /*#__PURE__*/React.createElement(FocusOn, {
      scrollLock: false,
      returnFocus: false,
      onDeactivation: handleReturnFocus,
      onClickOutside: function () {
        return setIsOpen(false);
      },
      onEscapeKey: function () {
        return setIsOpen(false);
      },
      enabled: isOpen
    }, /*#__PURE__*/React.createElement("div", {
      ref: containerRef
    }, /*#__PURE__*/React.createElement(DateInputField, __assign({
      ref: dateInputRefs,
      id: id,
      role: "combobox",
      "aria-haspopup": "dialog",
      "aria-expanded": isOpen,
      "aria-controls": calendarId,
      value: inputValue,
      locale: locale,
      onButtonClick: handleButtonClick,
      onClick: handleInputClick,
      onFocus: handleInputFocus,
      onChange: handleInputChange,
      onBlur: handleInputBlur,
      onKeyDown: handleKeyDown,
      status: status !== undefined || !shouldUseInbuiltValidation ? status : inbuiltStatus,
      validationMessage: validationMessage !== undefined || !shouldUseInbuiltValidation ? validationMessage : inbuiltValidationMessage
    }, restDateInputFieldProps))), isOpen && (/*#__PURE__*/React.createElement(CalendarPopover, {
      referenceElement: containerRef.current,
      "aria-labelledby": "".concat(id, "-calendar-label ").concat(id, "-input-label")
    }, /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(VisuallyHidden, {
      id: "".concat(id, "-calendar-label")
    }, calendarLabelDesc), /*#__PURE__*/React.createElement(CalendarSingle, {
      id: calendarId,
      selected: selectedDay,
      defaultMonth: defaultMonth,
      weekStartsOn: weekStartsOn,
      disabled: disabledDays,
      locale: locale,
      onDayClick: handleCalendarDayChange,
      onMount: handleCalendarMount
    })))));
  };
  DatePicker.displayName = 'DatePicker';
  return DatePicker;
}();
export { DatePicker };
